﻿using System;
using System.Collections;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Controls;
using Microsoft.SPOT.Presentation.Media;
using Microsoft.SPOT.Presentation.Shapes;
using Microsoft.SPOT.Touch;

using Gadgeteer.Networking;
using GT = Gadgeteer;
using GTM = Gadgeteer.Modules;
using Gadgeteer.Modules.GHIElectronics;

using Microsoft.SPOT.Time;
using System.Net; //System.Http, need to also add System.dll to references

namespace MSDNGadgeteerAzureBlob
{
    public partial class Program
    {
        // This method is run when the mainboard is powered up or reset.   
        void ProgramStarted()
        {
            /*******************************************************************************************
            Modules added in the Program.gadgeteer designer view are used by typing 
            their name followed by a period, e.g.  button.  or  camera.
            
            Many modules generate useful events. Type +=<tab><tab> to add a handler to an event, e.g.:
                button.ButtonPressed +=<tab><tab>
            
            If you want to do something periodically, use a GT.Timer and handle its Tick event, e.g.:
                GT.Timer timer = new GT.Timer(1000); // every second (1000ms)
                timer.Tick +=<tab><tab>
                timer.Start();
            *******************************************************************************************/


            // Use Debug.Print to show messages in Visual Studio's "Output" window during debugging.
            Debug.Print("Program Started");
            button.ButtonPressed += button_ButtonPressed;
            camera.PictureCaptured += camera_PictureCaptured;

            initializeNetwork();

            if (ethernetJ11D.IsNetworkUp)
            {
                SetTime();
                characterDisplay.Clear();
                characterDisplay.CursorHome();
                Microsoft.SPOT.Net.NetworkInformation.NetworkInterface settings = ethernetJ11D.NetworkSettings;
                characterDisplay.Print(DateTime.Now.ToString() + "\n");
                characterDisplay.Print("IP:" + settings.IPAddress);
                ListNetworkInterfaces();
            }
        }

        void button_ButtonPressed(Button sender, Button.ButtonState state)
        {
            if (camera.CameraReady)
            {
                camera.TakePicture();
            }
            else
            {
                Debug.Print("Camera not ready, no picture taken");
                //Add code to take from project directory, since Camera is broken sometimes
                
                GT.Picture picture = new GT.Picture(System.IO.File.ReadAllBytes("benperk.bmp"), GT.Picture.PictureEncoding.BMP);
                Thread.Sleep(500);
                Debug.Print("picture.PictureData.Length: " + picture.PictureData.Length.ToString());

                if (picture.PictureData.Length > 0)
                {
                    insertImageintoAzureBlob(picture);
                }
                else
                {
                    Debug.Print("NO IMAGE AVAILABLE FOR UPLOAD");
                }
            }
        }

        void initializeNetwork()
        {
            try
            {
                ethernetJ11D.NetworkInterface.Open();
                ethernetJ11D.NetworkSettings.EnableDhcp();
                ethernetJ11D.NetworkSettings.EnableDynamicDns();
                ethernetJ11D.UseStaticIP("192.168.1.222", "255.255.254.0", "192.168.1.1");
                ethernetJ11D.UseDHCP();
                Thread.Sleep(1250);

                while (ethernetJ11D.NetworkSettings.IPAddress == "192.168.1.222")
                {
                    Debug.Print("Waiting for DHCP");
                    Thread.Sleep(250);
                }

            }
            catch (Exception ex)
            {
                Debug.Print(ex.Message);
            }

            if (ethernetJ11D.IsNetworkUp)
            {
                multicolorLED.TurnGreen();
            }
            else
            {
                multicolorLED.BlinkRepeatedly(GT.Color.Red);
            }
        }

        void ListNetworkInterfaces()
        {
            var settings = ethernetJ11D.NetworkSettings;

            Debug.Print("------------------------------------------------");
            Debug.Print("MAC: " + ByteExtensions.ToHexString(settings.PhysicalAddress));
            Debug.Print("IP Address:   " + settings.IPAddress);
            Debug.Print("DHCP Enabled: " + settings.IsDhcpEnabled);
            Debug.Print("Subnet Mask:  " + settings.SubnetMask);
            Debug.Print("Gateway:      " + settings.GatewayAddress);
            Debug.Print("------------------------------------------------");
        }

        void SetTime()
        {
            Microsoft.SPOT.Time.TimeServiceSettings timeSettings = new Microsoft.SPOT.Time.TimeServiceSettings()
            {
                ForceSyncAtWakeUp = true
            };
            
            IPAddress[] address = System.Net.Dns.GetHostEntry("time.windows.com").AddressList;
            timeSettings.PrimaryServer = address[0].GetAddressBytes();
            TimeService.Settings = timeSettings;
            TimeService.SetTimeZoneOffset(0);
            TimeService.Start();
        }

        void camera_PictureCaptured(Camera sender, GT.Picture e)
        {
            insertImageintoAzureBlob(e);
        }

        void insertImageintoAzureBlob(GT.Picture picture)
        {
            AzureBlob storage = new AzureBlob()
            {
                Account = "put",
                BlobEndPoint = "https://put.blob.core.windows.net/",
                Key = "kkfS7GAPafEZmlJsQLJTBh2LKILtOIKHgAI44rjMoIbOE4QpS9o+FfTe/GCXAry8lOXX7pvFAfEBwtNrlmSPlg=="
            };
            
            if (ethernetJ11D.IsNetworkUp)
            {
                try
                {
                    storage.PutBlob("blob", picture.PictureData);
                }
                catch (Exception ex)
                {
                    Debug.Print("EXCEPTION: " + ex.Message);
                }
            }
            else
            {
                characterDisplay.Print("NO NETWORK CONNECTION");
            }
        }

    }
}
